@@ -1,7 +1,7 @@
 /* Trampoline construction */
 
 /*
- * Copyright 1995-1999, 2001 Bruno Haible, <haible@clisp.cons.org>
+ * Copyright 1995-1999, 2001-2003 Bruno Haible, <haible@clisp.cons.org>
  *
  * This is free software distributed under the GNU General Public Licence
  * described in the file COPYING. Contact the author if you don't have this
@@ -21,12 +21,16 @@
 #endif
 #endif
 #if defined(__rs6000__)
+#if defined(__NetBSD__)
+#define __rs6000netbsd__
+#else
 #if !defined(_AIX)
 #define __rs6000sysv4__  /* SysV.4 ABI, real machine code. */
 #else
 #define __rs6000aix__  /* AIX ABI, just a closure. */
 #endif
 #endif
+#endif
 #if defined(__hppanew__)
 /*
  * A function pointer is a biased pointer to a data area whose first word
@@ -91,39 +95,16 @@ extern void (*tramp_r) (); /* trampoline prototype */
 #include <stdio.h> /* declares fprintf() */
 
 #include <sys/types.h>
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
+#include <stdlib.h> /* declares abort(), malloc(), free() */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-/* Declare malloc(), free(). */
-#ifndef malloc
-#if defined(__STDC__) || defined(__cplusplus)
-extern RETMALLOCTYPE malloc (MALLOC_SIZE_T size);
-#else
-extern RETMALLOCTYPE malloc();
-#endif
-#endif
-#ifndef free
-#if defined(__STDC__) || defined(__cplusplus)
-extern RETFREETYPE free (RETMALLOCTYPE ptr);
-#else
-extern RETFREETYPE free();
-#endif
-#endif
-
-/* Declare abort(). */
-#if defined(__STDC__) || defined(__cplusplus)
-extern ABORT_VOLATILE RETABORTTYPE abort (void);
-#else
-extern ABORT_VOLATILE RETABORTTYPE abort();
-#endif
-
 /* Declare getpagesize(). */
 #ifdef HAVE_GETPAGESIZE
-#if defined(__STDC__) || defined(__cplusplus)
+#ifdef __cplusplus
+extern "C" RETGETPAGESIZETYPE getpagesize (void);
+#elif defined(__STDC__)
 extern RETGETPAGESIZETYPE getpagesize (void);
 #else
 extern RETGETPAGESIZETYPE getpagesize ();
@@ -157,13 +138,6 @@ extern RETGETPAGESIZETYPE getpagesize ();
 #else
 #include <sys/types.h>
 #include <sys/mman.h>
-#if !defined(__convex__)
-#if defined(__STDC__) || defined(__cplusplus)
-extern int mprotect (MPROTECT_CONST MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot);
-#else
-extern int mprotect ();
-#endif
-#endif
 #endif
 #endif
 #endif
@@ -175,11 +149,6 @@ extern int mprotect ();
 #if !defined(PROT_EXEC) && defined(PROT_EXECUTE) /* Irix 4.0.5 needs this */
 #define PROT_EXEC PROT_EXECUTE
 #endif
-#if defined(__STDC__) || defined(__cplusplus)
-extern RETMMAPTYPE mmap (MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot, int flags, int fd, off_t off);
-#else
-extern RETMMAPTYPE mmap ();
-#endif
 #endif
 
 /* Declare open(). */
@@ -190,15 +159,6 @@ extern RETMMAPTYPE mmap ();
 #ifdef OPEN_NEEDS_SYS_FILE_H
 #include <sys/file.h>
 #endif
-#if defined(__STDC__) || defined(__cplusplus)
-#ifdef OPEN_DOTS
-extern int open (OPEN_CONST char* path, int flags, ...);
-#else
-extern int open (OPEN_CONST char* path, int flags, mode_t mode);
-#endif
-#else
-extern int open ();
-#endif
 #endif
 
 /* Declare shmget(), shmat(), shmctl(). */
@@ -209,19 +169,6 @@ extern int open ();
 #ifdef HAVE_SYS_SYSMACROS_H
 #include <sys/sysmacros.h>
 #endif
-#if defined(__STDC__) || defined(__cplusplus)
-extern int shmget (key_t key, SHMGET_SIZE_T size, int shmflg);
-extern RETSHMATTYPE shmat (int shmid, SHMAT_CONST RETSHMATTYPE shmaddr, int shmflg);
-#ifdef SHMCTL_DOTS
-extern int shmctl (int shmid, int cmd, ...);
-#else
-extern int shmctl (int shmid, int cmd, struct shmid_ds * buf);
-#endif
-#else
-extern int shmget ();
-extern RETSHMATTYPE shmat ();
-extern int shmctl ();
-#endif
 #endif
 
 /* Support for instruction cache flush. */
@@ -247,7 +194,7 @@ extern int shmctl ();
 #include <mips/cachectl.h>
 #else
 #ifdef linux
-#include <linux/cachectl.h>
+#include <asm/cachectl.h>
 #else
 #ifdef HAVE_SYS_CACHECTL_H
 #include <sys/cachectl.h>
@@ -259,7 +206,7 @@ extern int shmctl ();
 #include <sys/syslocal.h>
 #endif
 /* Inline assembly function for instruction cache flush. */
-#if defined(__sparc__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppaold__) || defined(__rs6000sysv4__) || defined(__convex__)
+#if defined(__sparc__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppaold__) || defined(__rs6000sysv4__) || defined(__rs6000netbsd__) || defined(__convex__)
 #ifdef __GNUC__
 extern inline
 #if defined(__sparc__) || defined(__sparc64__)
@@ -340,6 +287,10 @@ extern void __TR_clear_cache();
 #define TRAMP_LENGTH 24
 #define TRAMP_ALIGN 4
 #endif
+#ifdef __rs6000netbsd__
+#define TRAMP_LENGTH 24
+#define TRAMP_ALIGN 4
+#endif
 #ifdef __rs6000aix__
 #define TRAMP_LENGTH 20
 #define TRAMP_ALIGN 4
@@ -356,6 +307,10 @@ extern void __TR_clear_cache();
 #define TRAMP_LENGTH 32
 #define TRAMP_ALIGN 16
 #endif
+#ifdef __s390__
+#define TRAMP_LENGTH 22
+#define TRAMP_ALIGN 2
+#endif
 
 #ifndef TRAMP_BIAS
 #define TRAMP_BIAS 0
@@ -560,6 +515,37 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #define tramp_data(function)  \
   *(unsigned int *) (function +16)
 #endif
+#ifdef __s390__
+  /* function:
+
+        bras    %r1,.LTN0_0
+.LT0_0:
+.LC0:
+        .long   0x73554711
+.LC1:
+        .long   0xbabebec0
+.LTN0_0:
+        l       %r0,.LC0-.LT0_0(%r1)
+        l       %r1,.LC1-.LT1_0(%r1)
+        br      %r1
+  */
+  /* What about big endian / little endian ?? */
+  *(unsigned int *)   (function + 0) = 0xA7150006;
+  *(unsigned int *)   (function + 4) = (unsigned int) data;
+  *(unsigned int *)   (function + 8) = (unsigned int) address;
+  *(unsigned int *)   (function +12) = 0x58001000;
+  *(unsigned int *)   (function +16) = 0x58101004;
+  *(unsigned short *) (function +20) = 0x07f1;
+#define is_tramp(function)  \
+  *(int *)            (function + 0) == 0xA7150006 && \
+  *(int *)            (function +12) == 0x58001000 && \
+  *(int *)            (function +16) == 0x58101004 && \
+  *(short *)          (function +20) == 0x07f1
+#define tramp_address(function)  \
+  *(unsigned int *) (function +8)
+#define tramp_data(function)  \
+  *(unsigned int *) (function +4)
+#endif
 #ifdef __mips64old__
   /* function:
    *    dli $2,<data>			3C 02 hi16(hi32(<data>))
@@ -805,12 +791,15 @@ __TR_function alloc_trampoline_r (address, data0, data1)
    *    .long   <data>
    *    .long   <address>
    */
-  *(long *) (function + 0) = ((long *) ((char*)&tramp_r-2))[0];
-  *(long *) (function + 4) = (long) (function + 8);
-  *(long *) (function + 8) = (long) data;
-  *(long *) (function +12) = (long) address;
+  { /* work around a bug in gcc 3.* */
+    void* tramp_r_address = &tramp_r;
+    *(long *) (function + 0) = ((long *) ((char*)tramp_r_address-2))[0];
+    *(long *) (function + 4) = (long) (function + 8);
+    *(long *) (function + 8) = (long) data;
+    *(long *) (function +12) = (long) address;
+  }
 #define is_tramp(function)  \
-  ((long *) function)[0] == ((long *) ((char*)&tramp_r-2))[0]
+  ((long *) function)[0] == ((long *) ((char*)tramp_r_address-2))[0]
 #define tramp_address(function)  \
   ((long *) function)[3]
 #define tramp_data(function)  \
@@ -883,6 +872,39 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #define tramp_data(function)  \
   hilo(*(unsigned short *) (function + 2), *(unsigned short *) (function + 6))
 #endif
+#ifdef __rs6000netbsd__
+  /* function:
+   *    {liu|lis} 13,hi16(<data>)		3D A0 hi16(<data>)
+   *    {oril|ori} 13,13,lo16(<data>)		61 AD lo16(<data>)
+   *    {liu|lis} 0,hi16(<address>)		3C 00 hi16(<address>)
+   *    {oril|ori} 0,0,lo16(<address>)		60 00 lo16(<address>)
+   *    mtctr 0					7C 09 03 A6
+   *    bctr					4E 80 04 20
+   */
+  *(short *) (function + 0) = 0x3DA0;
+  *(short *) (function + 2) = (unsigned long) data >> 16;
+  *(short *) (function + 4) = 0x61AD;
+  *(short *) (function + 6) = (unsigned long) data & 0xffff;
+  *(short *) (function + 8) = 0x3C00;
+  *(short *) (function +10) = (unsigned long) address >> 16;
+  *(short *) (function +12) = 0x6000;
+  *(short *) (function +14) = (unsigned long) address & 0xffff;
+  *(long *)  (function +16) = 0x7C0903A6;
+  *(long *)  (function +20) = 0x4E800420;
+#define is_tramp(function)  \
+  *(unsigned short *) (function + 0) == 0x3DA0 && \
+  *(unsigned short *) (function + 4) == 0x61AD && \
+  *(unsigned short *) (function + 8) == 0x3C00 && \
+  *(unsigned short *) (function +12) == 0x6000 && \
+  *(unsigned long *)  (function +16) == 0x7C0903A6 && \
+  *(unsigned long *)  (function +20) == 0x4E800420
+#define hilo(hiword,loword)  \
+  (((unsigned long) (hiword) << 16) | (unsigned long) (loword))
+#define tramp_address(function)  \
+  hilo(*(unsigned short *) (function +10), *(unsigned short *) (function +14))
+#define tramp_data(function)  \
+  hilo(*(unsigned short *) (function + 2), *(unsigned short *) (function + 6))
+#endif
 #ifdef __rs6000aix__
   /* function:
    *    .long .tramp_r
@@ -1001,7 +1023,7 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #if defined(HAVE_SYS_M88KBCS_H)
     if (memctl(start_addr, len, MCT_TEXT) == -1)
 #else
-    if (mprotect((MMAP_ADDR_T)start_addr, len, PROT_READ|PROT_WRITE|PROT_EXEC) < 0)
+    if (mprotect((void*)start_addr, len, PROT_READ|PROT_WRITE|PROT_EXEC) < 0)
 #endif
 #endif
 #endif
@@ -1139,6 +1161,7 @@ int is_trampoline_r (function)
 {
 #if defined(is_tramp) && defined(tramp_data)
 #ifdef __hppanew__
+  void* tramp_r_address = &tramp_r;
   if (!(((long)function & 3) == (TRAMP_BIAS & 3))) return 0;
 #endif
   return
