@@ -1,7 +1,7 @@
 /* Trampoline construction */
 
 /*
- * Copyright 1995-1999, 2001-2002 Bruno Haible, <haible@clisp.cons.org>
+ * Copyright 1995-1999, 2001-2003 Bruno Haible, <haible@clisp.cons.org>
  *
  * This is free software distributed under the GNU General Public Licence
  * described in the file COPYING. Contact the author if you don't have this
@@ -91,42 +91,11 @@ extern void (*tramp) (); /* trampoline prototype */
 #include <stdio.h> /* declares fprintf() */
 
 #include <sys/types.h>
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
+#include <stdlib.h> /* declares abort(), malloc(), free() */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-/* Declare malloc(), free(). */
-#ifndef malloc
-#ifdef __cplusplus
-extern "C" RETMALLOCTYPE malloc (MALLOC_SIZE_T size);
-#elif defined(__STDC__)
-extern RETMALLOCTYPE malloc (MALLOC_SIZE_T size);
-#else
-extern RETMALLOCTYPE malloc();
-#endif
-#endif
-#ifndef free
-#ifdef __cplusplus
-extern "C" RETFREETYPE free (RETMALLOCTYPE ptr);
-#elif defined(__STDC__)
-extern RETFREETYPE free (RETMALLOCTYPE ptr);
-#else
-extern RETFREETYPE free();
-#endif
-#endif
-
-/* Declare abort(). */
-#ifdef __cplusplus
-extern "C" ABORT_VOLATILE RETABORTTYPE abort (void);
-#elif defined(__STDC__)
-extern ABORT_VOLATILE RETABORTTYPE abort (void);
-#else
-extern ABORT_VOLATILE RETABORTTYPE abort();
-#endif
-
 /* Declare getpagesize(). */
 #ifdef HAVE_GETPAGESIZE
 #ifdef __cplusplus
@@ -165,15 +134,6 @@ extern RETGETPAGESIZETYPE getpagesize ();
 #else
 #include <sys/types.h>
 #include <sys/mman.h>
-#if !defined(__convex__)
-#ifdef __cplusplus
-extern "C" int mprotect (MPROTECT_CONST MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot);
-#elif defined(__STDC__)
-extern int mprotect (MPROTECT_CONST MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot);
-#else
-extern int mprotect ();
-#endif
-#endif
 #endif
 #endif
 #endif
@@ -185,13 +145,6 @@ extern int mprotect ();
 #if !defined(PROT_EXEC) && defined(PROT_EXECUTE) /* Irix 4.0.5 needs this */
 #define PROT_EXEC PROT_EXECUTE
 #endif
-#ifdef __cplusplus
-extern "C" RETMMAPTYPE mmap (MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot, int flags, int fd, off_t off);
-#elif defined(__STDC__)
-extern RETMMAPTYPE mmap (MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot, int flags, int fd, off_t off);
-#else
-extern RETMMAPTYPE mmap ();
-#endif
 #endif
 
 /* Declare open(). */
@@ -202,21 +155,6 @@ extern RETMMAPTYPE mmap ();
 #ifdef OPEN_NEEDS_SYS_FILE_H
 #include <sys/file.h>
 #endif
-#ifdef __cplusplus
-#ifdef OPEN_DOTS
-extern "C" int open (OPEN_CONST char* path, int flags, ...);
-#else
-extern "C" int open (OPEN_CONST char* path, int flags, mode_t mode);
-#endif
-#elif defined(__STDC__)
-#ifdef OPEN_DOTS
-extern int open (OPEN_CONST char* path, int flags, ...);
-#else
-extern int open (OPEN_CONST char* path, int flags, mode_t mode);
-#endif
-#else
-extern int open ();
-#endif
 #endif
 
 /* Declare shmget(), shmat(), shmctl(). */
@@ -227,27 +165,6 @@ extern int open ();
 #ifdef HAVE_SYS_SYSMACROS_H
 #include <sys/sysmacros.h>
 #endif
-#ifdef __cplusplus
-extern "C" int shmget (key_t key, SHMGET_SIZE_T size, int shmflg);
-extern "C" RETSHMATTYPE shmat (int shmid, SHMAT_CONST RETSHMATTYPE shmaddr, int shmflg);
-#ifdef SHMCTL_DOTS
-extern "C" int shmctl (int shmid, int cmd, ...);
-#else
-extern "C" int shmctl (int shmid, int cmd, struct shmid_ds * buf);
-#endif
-#elif defined(__STDC__)
-extern int shmget (key_t key, SHMGET_SIZE_T size, int shmflg);
-extern RETSHMATTYPE shmat (int shmid, SHMAT_CONST RETSHMATTYPE shmaddr, int shmflg);
-#ifdef SHMCTL_DOTS
-extern int shmctl (int shmid, int cmd, ...);
-#else
-extern int shmctl (int shmid, int cmd, struct shmid_ds * buf);
-#endif
-#else
-extern int shmget ();
-extern RETSHMATTYPE shmat ();
-extern int shmctl ();
-#endif
 #endif
 
 /* Support for instruction cache flush. */
@@ -273,7 +190,7 @@ extern int shmctl ();
 #include <mips/cachectl.h>
 #else
 #ifdef linux
-#include <linux/cachectl.h>
+#include <asm/cachectl.h>
 #else
 #ifdef HAVE_SYS_CACHECTL_H
 #include <sys/cachectl.h>
@@ -615,25 +532,25 @@ __TR_function alloc_trampoline (address, variable, data)
   */
   /* What about big endian / little endian ?? */
   *(unsigned short *) (function + 0) = 0x180D;
-  *(unsigned int *) (function + 2) = 0xA7D50008;
-  *(unsigned int *) (function + 6) = (unsigned int) variable;
-  *(unsigned int *) (function +10) = (unsigned int) data;
-  *(unsigned int *) (function +14) = (unsigned int) address;
-  *(unsigned int *) (function +18) = 0x5810D000;
-  *(unsigned int *) (function +22) = 0xd2031000;
+  *(unsigned int *)   (function + 2) = 0xA7D50008;
+  *(unsigned int *)   (function + 6) = (unsigned int) variable;
+  *(unsigned int *)   (function +10) = (unsigned int) data;
+  *(unsigned int *)   (function +14) = (unsigned int) address;
+  *(unsigned int *)   (function +18) = 0x5810D000;
+  *(unsigned int *)   (function +22) = 0xD2031000;
   *(unsigned short *) (function +26) = 0xD004;
-  *(unsigned int *) (function +28) = 0x5810D008;
+  *(unsigned int *)   (function +28) = 0x5810D008;
   *(unsigned short *) (function +32) = 0x18D0;
   *(unsigned short *) (function +34) = 0x07f1;
 #define is_tramp(function)  \
-  *(unsigned short *)   (function + 0) == 0x180D && \
-  *(unsigned int *)     (function + 2) == 0xA7D50008 && \
-  *(unsigned int *)     (function +18) == 0x5810D000 && \
-  *(unsigned int *)     (function +22) == 0xd2031000 && \
-  *(unsigned short *)   (function +26) == 0xD004 && \
-  *(unsigned int *)     (function +28) == 0x5810D008 && \
-  *(unsigned short *)   (function +32) == 0x18D0 && \
-  *(unsigned short *)   (function +34) == 0x07f1
+  *(short *)          (function + 0) == 0x180D && \
+  *(int *)            (function + 2) == 0xA7D50008 && \
+  *(int *)            (function +18) == 0x5810D000 && \
+  *(int *)            (function +22) == 0xD2031000 && \
+  *(short *)          (function +26) == 0xD004 && \
+  *(int *)            (function +28) == 0x5810D008 && \
+  *(short *)          (function +32) == 0x18D0 && \
+  *(short *)          (function +34) == 0x07f1
 #define tramp_address(function)  \
   *(unsigned int *) (function +14)
 #define tramp_variable(function)  \
@@ -963,13 +880,16 @@ __TR_function alloc_trampoline (address, variable, data)
    *    .long   <data>
    *    .long   <address>
    */
-  *(long *) (function + 0) = ((long *) ((char*)&tramp-2))[0];
-  *(long *) (function + 4) = (long) (function + 8);
-  *(long *) (function + 8) = (long) variable;
-  *(long *) (function +12) = (long) data;
-  *(long *) (function +16) = (long) address;
+  { /* work around a bug in gcc 3.* */
+    void* tramp_address = &tramp;
+    *(long *) (function + 0) = ((long *) ((char*)tramp_address-2))[0];
+    *(long *) (function + 4) = (long) (function + 8);
+    *(long *) (function + 8) = (long) variable;
+    *(long *) (function +12) = (long) data;
+    *(long *) (function +16) = (long) address;
+  }
 #define is_tramp(function)  \
-  ((long *) function)[0] == ((long *) ((char*)&tramp-2))[0]
+  ((long *) function)[0] == ((long *) ((char*)tramp_address-2))[0]
 #define tramp_address(function)  \
   ((long *) function)[4]
 #define tramp_variable(function)  \
@@ -1204,7 +1124,7 @@ __TR_function alloc_trampoline (address, variable, data)
 #if defined(HAVE_SYS_M88KBCS_H)
     if (memctl(start_addr, len, MCT_TEXT) == -1)
 #else
-    if (mprotect((MMAP_ADDR_T)start_addr, len, PROT_READ|PROT_WRITE|PROT_EXEC) < 0)
+    if (mprotect((void*)start_addr, len, PROT_READ|PROT_WRITE|PROT_EXEC) < 0)
 #endif
 #endif
 #endif
@@ -1342,6 +1262,7 @@ int is_trampoline (function)
 {
 #ifdef is_tramp
 #ifdef __hppanew__
+  void* tramp_address = &tramp;
   if (!(((long)function & 3) == (TRAMP_BIAS & 3))) return 0;
 #endif
   return ((is_tramp(((char*)function - TRAMP_BIAS))) ? 1 : 0);
