@@ -1,6 +1,6 @@
 ;;; Foreign function interface for CLISP
 ;;; Bruno Haible 19.2.1995
-;;; Sam Steingold 1998-2002
+;;; Sam Steingold 1998-2003
 
 #+UNICODE
 (progn
@@ -11,20 +11,22 @@
 (use-package '("COMMON-LISP" "EXT") "FFI")
 (in-package "FFI")
 
-(export '(def-c-type def-c-var
+(export '(def-c-type def-c-var parse-c-type deparse-c-type
           def-c-call-out def-call-out #+AMIGA def-lib-call-out
           def-c-call-in def-call-in default-foreign-language
-          c-lines
+          c-lines *output-c-functions* *output-c-variables*
           nil boolean character char uchar short ushort int uint long ulong
           uint8 sint8 uint16 sint16 uint32 sint32 uint64 sint64
           single-float double-float
           c-pointer c-string c-struct c-union c-array c-array-max
           c-function c-ptr c-ptr-null c-array-ptr
           def-c-enum def-c-struct element deref slot cast typeof
-          sizeof bitsizeof c-var-address offset
-          validp foreign-address-null
+          sizeof bitsizeof c-var-object c-var-address offset
+          validp with-c-place foreign-value enum-from-value enum-to-value
+          foreign-address foreign-address-unsigned unsigned-foreign-address
           with-foreign-object with-c-var with-foreign-string
-          foreign-address foreign-variable foreign-function))
+          foreign-allocate allocate-deep allocate-shallow foreign-free
+          foreign-pointer foreign-variable foreign-function))
 
 (eval-when (load compile eval)
   (import (intern "*COUTPUT-FILE*" "COMPILER"))
@@ -33,7 +35,6 @@
   (import (intern "FINALIZE-COUTPUT-FILE" "COMPILER"))
   (import (intern "TEXT" "SYSTEM")) ; messages
   (import (intern "CHECK-SYMBOL" "SYSTEM")) ; error checking
-  (import (intern "DEPARSE-C-TYPE" "SYSTEM")) ; called by DESCRIBE
   (import (intern "FOREIGN-FUNCTION-IN-ARG-COUNT" "SYSTEM")) ; called by SYS::FUNCTION-SIGNATURE
 )
 
@@ -53,6 +54,13 @@
   (defvar ff-language-stdcall)
 )
 
+;; ===========================================================================
+
+#+UNICODE
+(progn
+  (define-symbol-macro *foreign-encoding* (system::foreign-encoding))
+  (defsetf system::foreign-encoding system::set-foreign-encoding))
+
 ;; ============================ helper functions ============================
 
 ; Determines whether a name is a valid C identifier.
@@ -121,7 +129,7 @@
   (let* ((form-type (first typespec))
          (spec-list
           (ecase form-type
-            (C-STRUCT (cddr typespec))
+            (C-STRUCT (cddr typEspec))
             (C-UNION (cdr typespec)))))
     (mapcar (lambda (subspec)
               (unless (and (consp subspec)
@@ -135,6 +143,7 @@
 ;; return the constructor function for this structure
 (defun c-struct-constructor (typespec)
   (let ((class (second typespec)))
+    (when (consp class) (setq class (car class)))  ; name+options -> name
     (case class
       (VECTOR #'vector)
       (LIST #'list)
@@ -162,6 +171,14 @@
                                        slots vars)
                              ,ivar)))))))))))
 
+(defmacro with-name/options ((name options name+options) &body body)
+  (let ((no (gensym "NAME+OPTIONS-")))
+    `(let ((,no ,name+options))
+       (multiple-value-bind (,name ,options)
+           (if (consp ,no) (values (first ,no) (rest ,no)) (values ,no nil))
+         (declare (ignorable ,name ,options))
+         ,@body))))
+
 (defmacro with-defining-c-type ((name c-type len) &body body)
   `(let ((,c-type (make-array ,len)))
     (unwind-protect
@@ -190,12 +207,15 @@
            (dimp (dim) (typep dim '(integer 0 *))))
       (case (first typespec)
         (C-STRUCT
-         (with-defining-c-type (name c-type (1+ (length typespec)))
-           (setf (svref c-type 0) (first typespec))
-           (setf (svref c-type 1) (map 'vector #'first (cddr typespec)))
-           (setf (svref c-type 2) ; constructor
+         (with-defining-c-type (name c-type (max 5 (+ 3 (length typespec))))
+           (setf (svref c-type 0) (first typespec)) ; c-struct
+           (with-name/options (name options (second typespec))
+             (setf (svref c-type 1) name)
+             (setf (svref c-type 2) options))
+           (setf (svref c-type 3) (map 'vector #'first (cddr typespec)))
+           (setf (svref c-type 4) ; constructor
                  (c-struct-constructor typespec))
-           (setf (subseq c-type 3) (parse-components typespec))))
+           (setf (subseq c-type 5) (parse-components typespec))))
         (C-UNION
          (with-defining-c-type (name c-type (1+ (length typespec)))
            (setf (svref c-type 0) (first typespec))
@@ -282,7 +302,8 @@
 
 (defmacro default-foreign-language (lang)
   (language-to-flag lang)       ; error checking
-  `(eval-when (load compile eval) (setq *foreign-language* ',lang)))
+  `(eval-when (load compile eval)
+     (without-package-lock ("FFI") (setq *foreign-language* ',lang))))
 
 ;; get the even (start=0) or odd (start=1) elements of the simple vector
 (defun split-c-fun-arglist (args start)
@@ -360,8 +381,8 @@
      (PARSE-C-TYPE ',typespec ',name)
      ',name))
 
-; Convert back a C type from internal (vector) to external (list)
-; representation. Both representations may be circular.
+;; Convert back a C type from internal (vector) to external (list)
+;; representation. Both representations may be circular.
 (defun deparse-c-type (ctype)
   (let ((alist '()))
     (labels ((new-type (ctype typespec)
@@ -378,15 +399,20 @@
                        (new-type ctype typespec)
                        (setf (rest typespec) ; fill the rest
                              (ecase (svref ctype 0)
-                               ;; #(c-struct slots constructor <c-type>*)
+                               ;; #(c-struct name options slots
+                               ;;            constructor <c-type>*)
                                (C-STRUCT
-                                (cons
-                                 (let ((constructor (svref ctype 2)))
-                                   (cond ((eql constructor #'vector) 'vector)
-                                         ((eql constructor #'list) 'list)
-                                         (t 'nil)))
-                                 (map 'list #'deparse-slot
-                                      (svref ctype 1) (subseq ctype 3))))
+                                (let* ((constructor (svref ctype 4))
+                                       (options (svref ctype 2))
+                                       (name (cond ((eql constructor #'vector)
+                                                    'vector)
+                                                   ((eql constructor #'list)
+                                                    'list)
+                                                   (t (svref ctype 1)))))
+                                  (cons (if options (cons name options) name)
+                                        (map 'list #'deparse-slot
+                                             (svref ctype 3)
+                                             (subseq ctype 5)))))
                                ;; #(c-union alternatives <c-type>*)
                                (C-UNION
                                 (map 'list #'deparse-slot
@@ -531,22 +557,26 @@
     (c-pointer (format nil "void* ~A" name))
     (t (if (gethash c-type *type-table*)
          (format nil "~A ~A" (gethash c-type *type-table*) name)
-         (macrolet ((with-to-c ((class typename) &body body)
-                      `(let ((,typename
-                              (format nil "~A ~A" ,class (gensym "t"))))
-                        (unwind-protect
-                             (progn (setf (gethash c-type *type-table*)
-                                          ,typename)
-                                    ,@body)
-                          (setf (gethash c-type *type-table*) nil)))))
+         (macrolet ((with-to-c ((class typename &key (tname '(gensym "t")))
+                                &body body)
+                      `(let ((,typename (format nil "~A ~A" ,class ,tname)))
+                         (unwind-protect
+                              (progn (setf (gethash c-type *type-table*)
+                                           ,typename)
+                                     ,@body)
+                           (setf (gethash c-type *type-table*) nil)))))
            (case (ctype-type c-type)
              (c-struct
-              (with-to-c ("struct" type)
-                (format nil "~a { ~{~A; ~}} ~A"
-                        type (map 'list #'to-c-typedecl
-                                  (cdddr (coerce c-type 'list))
-                                  (svref c-type 1))
-                        name)))
+              (cond ((sys::memq :typedef (svref c-type 2))
+                     (format nil "~A ~A" (svref c-type 1) name))
+                    ((sys::memq :external (svref c-type 2))
+                     (format nil "struct ~A ~A" (svref c-type 1) name))
+                    (t (with-to-c ("struct" type :tname (svref c-type 1))
+                         (format nil "~a { ~{~A; ~}} ~A"
+                                 type (map 'list #'to-c-typedecl
+                                           (subseq c-type 5)
+                                           (svref c-type 3))
+                                 name)))))
              (c-union
               (with-to-c ("union" type)
                 (format nil "~A { ~{~A; ~}} ~A"
@@ -574,6 +604,9 @@
              (t (error (TEXT "illegal foreign data type ~S")
                        c-type))))))))
 
+(defvar *output-c-functions* nil)
+(defvar *output-c-variables* nil)
+
 (defun prepare-module ()
   (unless *ffi-module*
     (setq *ffi-module*
@@ -584,31 +617,26 @@
             *c-name*)))
 (defun finalize-coutput-file ()
   (when *ffi-module*
-    (format *coutput-stream* "~%")
-    (format *coutput-stream* "subr_t module__~A__subr_tab[1];~%" *c-name*)
-    (format *coutput-stream* "uintC module__~A__subr_tab_size = 0;~%" *c-name*)
-    (format *coutput-stream* "subr_initdata_t module__~A__subr_tab_initdata[1];~%"
-            *c-name*)
-    (format *coutput-stream* "~%")
+    (format *coutput-stream* "~%subr_t module__~A__subr_tab[1];~%~
+            uintC module__~A__subr_tab_size = 0;~%~
+            subr_initdata_t module__~A__subr_tab_initdata[1];~2%"
+            *c-name* *c-name* *c-name*)
     (let ((count (hash-table-count *object-table*)))
       (if (zerop count)
-        (progn
-          (format *coutput-stream* "object module__~A__object_tab[1];~%" *c-name*)
-          (format *coutput-stream*
-                  "object_initdata_t module__~A__object_tab_initdata[1];~%" *c-name*))
+        (format *coutput-stream* "object module__~A__object_tab[1];~%~
+                object_initdata_t module__~A__object_tab_initdata[1];~%"
+                *c-name* *c-name*)
         (let ((v (make-array count)))
-          (format *coutput-stream* "object module__~A__object_tab[~D];~%"
-                  *c-name* count)
-          (format *coutput-stream*
-                  "object_initdata_t module__~A__object_tab_initdata[~D] = {~%"
-                  *c-name* count)
+          (format *coutput-stream* "object module__~A__object_tab[~D];~%~
+                  object_initdata_t module__~A__object_tab_initdata[~D] = {~%"
+                  *c-name* count *c-name* count)
           (dohash (key value *object-table*)
             (declare (ignore key))
             (setf (svref v (cdr value)) (car value)))
           (map nil #'(lambda (initstring)
                        (format *coutput-stream* "  { ~A },~%"
                                (to-c-string initstring)))
-                   v)
+               v)
           (format *coutput-stream* "};~%")))
       (format *coutput-stream* "uintC module__~A__object_tab_size = ~D;~%"
               *c-name* count))
@@ -616,31 +644,42 @@
     (setq *variable-list*
           (nreverse (delete-duplicates
                      *variable-list* :key #'first :test #'equal)))
-    (dolist (variable *variable-list*)
-      ;(prepare-c-typedecl (second variable))
-      (format *coutput-stream* "extern ~A;~%"
-              (to-c-typedecl (second variable) (first variable))))
+    (when *output-c-variables*
+      (dolist (variable *variable-list*)
+        ;;(prepare-c-typedecl (second variable))
+        (format *coutput-stream* "extern ~A;~%"
+                (to-c-typedecl (second variable) (first variable)))))
     (setq *function-list*
           (nreverse (delete-duplicates
                      *function-list* :key #'first :test #'equal)))
-    (dolist (function *function-list*)
-      ;(prepare-c-typedecl (svref (second function) 1))
-      (format *coutput-stream* "extern ~A;~%"
-              (to-c-typedecl (svref (second function) 1)
-                             (format nil "(~A)()" (first function)))))
-    (format *coutput-stream* "~%void module__~A__init_function_1(module)
-  var module_t* module;~%{ }~%"
-            *c-name*)
-    (format *coutput-stream* "~%void module__~A__init_function_2(module)
-  var module_t* module;~%{~%"
-            *c-name*)
+    (when *output-c-functions*
+      (dolist (function *function-list*)
+        ;;(prepare-c-typedecl (svref (second function) 1))
+        (format *coutput-stream* "extern ~A"
+                (to-c-typedecl (svref (second function) 1)
+                               (format nil "(~A)(" (first function))))
+        (when (third function)    ; built-in, requires full arglist
+          (do* ((parameters (svref (second function) 2))
+                (length (length parameters)) (i 0 (+ 2 i))
+                (parameter (svref parameters i)))
+               ((>= i length))
+            (unless (zerop i)
+              (write-string ", " *coutput-stream*))
+            (write-string (to-c-typedecl parameter "")
+                          *coutput-stream*)))
+        (format *coutput-stream* ");~%")))
+    (format *coutput-stream*
+            "~%void module__~A__init_function_1 (module_t* module)~%{ }~2%~
+            void module__~A__init_function_2 (module_t* module)~%{~%"
+            *c-name* *c-name*)
     (dolist (variable *variable-list*)
       (format *coutput-stream*
-              "  register_foreign_variable(&~A,~A,~D,sizeof(~A));~%"
+              "  register_foreign_variable((void*)&~A,~A,~D,sizeof(~A));~%"
               (first variable) (to-c-string (first variable))
               (third variable) (first variable)))
     (dolist (function *function-list*)
-      (format *coutput-stream* "  register_foreign_function(&~A,~A,~D);~%"
+      (format *coutput-stream*
+              "  register_foreign_function((void*)&~A,~A,~D);~%"
               (first function) (to-c-string (first function))
               (svref (second function) 3)))
     (format *coutput-stream* "}~%")))
@@ -684,7 +723,9 @@
 
 (defmacro DEF-C-VAR (&whole whole name &rest options)
   (check-symbol (first whole) name)
-  (let* ((alist (parse-options options '(:name :type :read-only :alloc) whole))
+  (let* ((alist (parse-options options '(:name :type :read-only
+                                         :alloc :library)
+                               whole))
          (c-name (foreign-name name (assoc ':name alist)))
          (type (second (or (assoc ':type alist)
                            (sys::error-of-type 'sys::source-program-error
@@ -698,28 +739,34 @@
                          (:NONE 0)
                          (:MALLOC-FREE fv-flag-malloc-free))
                        0))))
+         (library  (second (assoc :library alist)))
          #|
          (getter-function-name (sys::symbol-suffix name "%GETTER%"))
          (setter-function-name (sys::symbol-suffix name "%SETTER%"))
          |#
         )
     `(PROGN
-       (EVAL-WHEN (COMPILE) (NOTE-C-VAR ',c-name ',type ',flags))
+       ,(unless library
+          `(EVAL-WHEN (COMPILE) (NOTE-C-VAR ',c-name ',type ',flags)))
        #|
-       (LET ((FVAR (FFI::LOOKUP-FOREIGN-VARIABLE ',c-name (PARSE-C-TYPE ',type))))
-         (DEFUN ,getter-function-name () (FFI::FOREIGN-VALUE FVAR))
+       (LET ((FVAR (LOOKUP-FOREIGN-VARIABLE ',c-name (PARSE-C-TYPE ',type))))
+         (DEFUN ,getter-function-name () (FOREIGN-VALUE FVAR))
          ; Install a setter even if the variable is read-only.
          ; When called, it will print a comprehensible error message.
-         (DEFUN ,setter-function-name (VALUE) (FFI::SET-FOREIGN-VALUE FVAR VALUE))
+         (DEFUN ,setter-function-name (VALUE) (SET-FOREIGN-VALUE FVAR VALUE))
        )
        (DEFSETF ,getter-function-name ,setter-function-name)
        (DEFINE-SYMBOL-MACRO ,name (,getter-function-name))
        |#
        (SYSTEM::%PUT ',name 'FOREIGN-VARIABLE
          (LOAD-TIME-VALUE
-           (FFI::LOOKUP-FOREIGN-VARIABLE ',c-name (PARSE-C-TYPE ',type))))
+           ,(if library
+              `(FFI::FOREIGN-LIBRARY-VARIABLE
+                ',c-name (FFI::FOREIGN-LIBRARY ',library)
+                nil (PARSE-C-TYPE ',type))
+              `(LOOKUP-FOREIGN-VARIABLE ',c-name (PARSE-C-TYPE ',type)))))
        (DEFINE-SYMBOL-MACRO ,name
-         (FFI::FOREIGN-VALUE (LOAD-TIME-VALUE (GET ',name 'FOREIGN-VARIABLE))))
+         (FOREIGN-VALUE (LOAD-TIME-VALUE (GET ',name 'FOREIGN-VARIABLE))))
        ',name)))
 
 (defun note-c-var (c-name type flags)
@@ -727,10 +774,14 @@
     (prepare-module)
     (push (list c-name (parse-c-type type) flags) *variable-list*)))
 
-(defsetf ffi::foreign-value ffi::set-foreign-value)
+(defsetf foreign-value set-foreign-value)
+(defsetf foreign-pointer set-foreign-pointer)
+(defsetf validp set-validp)
 
-(defun foreign-address-null (fadr)
-  (zerop (foreign-address-unsigned fadr)))
+(defmacro with-c-place ((var fvar) &body body)
+  (let ((fv (gensym (symbol-name var))))
+    `(LET ((,fv ,fvar))
+       (SYMBOL-MACROLET ((,var (FOREIGN-VALUE ,fv))) ,@body))))
 
 ;; ============================ Stack allocation ============================
 
@@ -748,18 +799,51 @@
 ;; using: `(c-array uint8 ,(length foo))
 (defmacro with-foreign-object ((var c-type &optional (init nil init-p))
                                &body body)
-  `(EXEC-ON-STACK (LAMBDA (,var) ,@body)
-                  (PARSE-C-TYPE ,c-type)
-                  . ,(if init-p (list init))))
+  `(EXEC-ON-STACK
+    (LAMBDA (,var) ,@body)
+    (PARSE-C-TYPE ,c-type)
+    . ,(if init-p (list init))))
 
 ;; symbol-macro based interface (like DEF-C-VAR)
+; WITH-C-VAR appears as a composition of WITH-FOREIGN-OBJECT and WITH-C-PLACE
 (defmacro with-c-var ((var c-type &optional (init nil init-p)) &body body)
   (let ((fv (gensym (symbol-name var))))
     `(EXEC-ON-STACK
-      (LAMBDA (,fv) (SYMBOL-MACROLET ((,var (FFI::FOREIGN-VALUE ,fv))) ,@body))
+      (LAMBDA (,fv) (SYMBOL-MACROLET ((,var (FOREIGN-VALUE ,fv))) ,@body))
       (PARSE-C-TYPE ,c-type)
       . ,(if init-p (list init)))))
 
+(defun exec-with-foreign-string (thunk string
+                                 &key (encoding #+UNICODE custom:*foreign-encoding*
+                                                #-UNICODE custom:*default-file-encoding*)
+                                      (null-terminated-p t) (start 0) (end nil))
+  (call-with-foreign-string
+   thunk encoding string start end
+   (if null-terminated-p
+       #-UNICODE 1
+       #+UNICODE (sys::encoding-zeroes encoding) 0)))
+
+(defmacro with-foreign-string ((foreign-variable char-count byte-count string
+                                &rest keywords
+                                &key encoding null-terminated-p start end)
+                               &body body)
+  (declare (ignore encoding null-terminated-p start end)) ; get them via keywords
+  `(exec-with-foreign-string
+    (lambda (,foreign-variable ,char-count ,byte-count) ,@body)
+    ,string .,keywords))
+
+;; ============================ heep allocation ============================
+
+(defmacro allocate-deep (ffi-type initval &rest keywords &key count read-only)
+  (declare (ignore count read-only)) ; to be accessed via keywords
+  `(foreign-allocate (parse-c-type ,ffi-type)
+                     :initial-contents ,initval
+                     ,@keywords))
+
+(defmacro allocate-shallow (ffi-type &rest keywords &key count read-only)
+  (declare (ignore count read-only)) ; to be accessed via keywords
+  `(foreign-allocate (parse-c-type ,ffi-type) ,@keywords))
+
 ;; ============================ named C functions ============================
 
 (defmacro DEF-C-CALL-OUT (name &rest options)
@@ -769,25 +853,35 @@
 
 (defmacro DEF-CALL-OUT (&whole whole name &rest options)
   (check-symbol (first whole) name)
-  (let* ((alist (parse-options options '(:name :arguments :return-type :language) whole))
+  (let* ((alist (parse-options options '(:name :arguments :return-type
+                                         :language :built-in :library)
+                               whole))
          (parsed-function (parse-c-function alist whole))
          (signature (argvector-to-signature (svref parsed-function 2)))
-         (c-name (foreign-name name (assoc ':name alist))))
-    (setq alist (remove (assoc ':name alist) alist))
+         (library (second (assoc :library alist)))
+         (c-name (foreign-name name (assoc :name alist))))
+    (setq alist (remove-if (lambda (el) (sys::memq (car el) '(:name :library)))
+                           alist))
     `(PROGN
-       (EVAL-WHEN (COMPILE) (NOTE-C-FUN ',c-name ',alist ',whole))
+       ,(unless library
+          `(EVAL-WHEN (COMPILE) (NOTE-C-FUN ',c-name ',alist ',whole)))
        (LET ()
          (SYSTEM::REMOVE-OLD-DEFINITIONS ',name)
          (COMPILER::EVAL-WHEN-COMPILE (COMPILER::C-DEFUN ',name ',signature))
          (SYSTEM::%PUTD ',name
-           (FFI::LOOKUP-FOREIGN-FUNCTION ',c-name
-                                         (PARSE-C-FUNCTION ',alist ',whole))))
+            ,(if library
+               `(ffi::foreign-library-function
+                 ',c-name (ffi::foreign-library ',library)
+                 nil ,parsed-function)
+               `(LOOKUP-FOREIGN-FUNCTION ',c-name ,parsed-function))))
        ',name)))
 
 (defun note-c-fun (c-name alist whole)
   (when (compiler::prepare-coutput-file)
     (prepare-module)
-    (push (list c-name (parse-c-function alist whole)) *function-list*)))
+    (push (list c-name (parse-c-function alist whole)
+                (cadr (assoc :built-in alist)))
+          *function-list*)))
 
 #+AMIGA
 (defmacro DEF-LIB-CALL-OUT (&whole whole name library &rest options)
@@ -802,8 +896,8 @@
        (SYSTEM::REMOVE-OLD-DEFINITIONS ',name)
        (COMPILER::EVAL-WHEN-COMPILE (COMPILER::C-DEFUN ',name ',signature))
        (SYSTEM::%PUTD ',name
-         (FFI::FOREIGN-LIBRARY-FUNCTION ',c-name
-           (FFI::FOREIGN-LIBRARY ',library)
+         (FOREIGN-LIBRARY-FUNCTION ',c-name
+           (FOREIGN-LIBRARY ',library)
            ',offset
            (PARSE-C-FUNCTION ',(remove (assoc ':name alist) alist) ',whole)))
        ',name)))
@@ -838,7 +932,7 @@
                              argtypes)))
       (prepare-c-typedecl rettype)
       ;(mapc #'prepare-c-typedecl argtypes)
-      (format *coutput-stream* "~%global ~A "
+      (format *coutput-stream* "~%extern ~A "
               (to-c-typedecl rettype (format nil "(~A)" c-name)))
       (if (flag-set-p flags ff-language-ansi-c)
         ; ANSI C parameter declarations
@@ -881,10 +975,9 @@
         (format *coutput-stream* "  funcall(~A,~D);~%"
                 (object-to-c-value (pass-object name)) inargcount)
         (unless (eq rettype 'NIL)
-          (format *coutput-stream* " {~%")
-          (format *coutput-stream* "  var ~A;~%"
-                  (to-c-typedecl rettype "retval"))
-          (format *coutput-stream* "  ~A(~A,value1,&retval);~%"
+          (format *coutput-stream* " {~%  var ~A;~%~:
+  ~A(~A,value1,&retval);~%"
+                  (to-c-typedecl rettype "retval")
                   (if (flag-set-p flags ff-flag-malloc-free)
                       "convert_to_foreign_mallocing"
                       "convert_to_foreign_nomalloc")
@@ -911,8 +1004,7 @@
                 argtypes argflags argnames))
         (format *coutput-stream* "  end_callback();~%")
         (unless (eq rettype 'NIL)
-          (format *coutput-stream* "  return retval;~%")
-          (format *coutput-stream* " }~%")))
+          (format *coutput-stream* "  return retval;~% }~%")))
       (format *coutput-stream* "}~%"))))
 
 ;; ===========================================================================
@@ -935,21 +1027,44 @@
 
 (defmacro def-c-enum (&whole whole name &rest items)
   (check-symbol (first whole) name)
-  (let ((forms '())
-        (next-value 0))
+  (let ((forms '()) (ht (make-hash-table))
+        (next-value 0) (this-val 0))
     (dolist (item items)
       (when (consp item)
-        (when (rest item) (setq next-value (second item)))
+        (when (rest item)
+          (setq next-value (second item)
+                this-val (second item)))
         (setq item (first item)))
       (push `(DEFCONSTANT ,item ,next-value) forms)
-      (setq next-value `(1+ ,item)))
-    (setf (gethash name *c-type-table*) 'int)
-    `(PROGN ,@(nreverse forms) ',name)))
+      (when (gethash this-val ht)
+        (warn (TEXT "~S (~S): value ~S will be assigned to both ~S and ~S")
+              'def-c-enum name this-val (gethash this-val ht) item ))
+      (setf (gethash this-val ht) item)
+      (setq next-value `(1+ ,item) this-val (1+ this-val)))
+    `(PROGN ,@(nreverse forms) (setf (get ',name 'def-c-enum) ,ht)
+            (def-c-type ,name int))))
 
-(defmacro def-c-struct (name &rest slots)
-  `(PROGN
-     (DEFSTRUCT ,name ,@(mapcar #'first slots))
-     (DEF-C-TYPE ,name (C-STRUCT ,name ,@slots))))
+(defun enum-table (enum)
+  (or (get enum 'def-c-enum)
+      (error (TEXT "~S does not name a C enum type") enum)))
+(defun enum-to-value (enum symbol)
+  (let ((val (dohash (k v (enum-table enum)
+                      (error (TEXT "~S is not of C enum type ~S") symbol enum))
+               (when (eq v symbol) (return k)))))
+    (unless (= val (symbol-value symbol))
+      (error
+       (TEXT "~S symbol value (~S) does not match its table value (~S) in ~S")
+       symbol (symbol-value symbol) val enum))
+    val))
+(defun enum-from-value (enum value)
+  (or (gethash value (enum-table enum))
+      (error (TEXT "~S is not a valid value of type ~S") value enum)))
+
+(defmacro def-c-struct (name+options &rest slots)
+  (with-name/options (name options name+options)
+    `(PROGN
+       (DEFSTRUCT ,name ,@(mapcar #'first slots))
+       (DEF-C-TYPE ,name (C-STRUCT ,name+options ,@slots)))))
 
 ;; In order for ELEMENT, DEREF, SLOT to be SETFable, I make them macros.
 ;; (element (foreign-value x) ...) --> (foreign-value (%element x ...))
@@ -988,6 +1103,12 @@
     (if (foreign-place-p place 'FOREIGN-VALUE)
       `(FOREIGN-VALUE (%OFFSET ,(second place) ,offset (PARSE-C-TYPE ,type)))
       (err `(offset ,place ,offset ,type))))
+  ;; Extract FOREIGN-VARIABLE object underlying the place
+  (defmacro c-var-object (place &environment env)
+    (setq place (macroexpand place env))
+    (if (foreign-place-p place 'FOREIGN-VALUE)
+      (second place)
+      (err `(c-var-object ,place))))
   ;; The equivalent of (FOREIGN-ADDRESS fvar) for c-places:
   ;; (c-var-address (foreign-value x)) --> (foreign-address x)
   (defmacro c-var-address (place &environment env)
@@ -1028,13 +1149,6 @@
 
 ;; ===========================================================================
 
-#+UNICODE
-(progn
-  (define-symbol-macro *foreign-encoding* (system::foreign-encoding))
-  (defsetf system::foreign-encoding system::set-foreign-encoding))
-
-;; ===========================================================================
-
 ;;; for self-test
 ;;; commented out since CLISP does not need them;
 ;;; they are defined by the test suite.
