@@ -280,7 +280,7 @@
    long-float-digits package-lock hash-table-weak-p weak-key-value-table
    the-environment arglist getenv special-variable-p list-length-dotted
    *driver* *break-driver* *args* *keyboard-input* clhs-root shell execute
-   define-hash-table-test
+   define-hash-table-test package-case-sensitive-p
    ;; declarations
    constant-inline constant-notinline
    ;; pseudo-types:
@@ -486,19 +486,6 @@
 (proclaim '(special *toplevel-denv*))
 (setq *toplevel-denv* (svref *toplevel-environment* 4))
 
-;; returns the name of the implicit block for a function-name
-(defun function-block-name (funname)
-  (if (atom funname) funname (second funname)))
-
-;; inserts an implicit BLOCK in the BODY.
-;; uses *VENV* and *FENV*.
-(defun add-implicit-block (name body)
-  (multiple-value-bind (body-rest declarations docstring)
-      (sys::parse-body body t (vector *venv* *fenv*))
-    (append (if declarations (list (cons 'DECLARE declarations)))
-            (if docstring (list docstring))
-            (list (list* 'BLOCK (function-block-name name) body-rest)))))
-
 ;;; functions for expansion of macros within a piece of code
 ;;;
 ;;; Altogether, the whole Code (of a function) is walked through and
@@ -618,11 +605,11 @@
     (let (h)
       (if (and (symbolp form)
                (symbol-macro-p (setq h (venv-assoc form *venv*))))
-        (values (sys::%record-ref h 0) t)
+        (values (%expand-form (sys::%record-ref h 0)) t)
         (values form nil)))
     #-COMPILER
     (if (and (symbolp form) (symbol-macro-p (venv-assoc form *venv*)))
-      (values (sys::%record-ref (venv-assoc form *venv*) 0) t)
+      (values (%expand-form (sys::%record-ref (venv-assoc form *venv*) 0)) t)
       (values form nil))
     ;; form is a CONS
     (let ((f (first form)))
@@ -946,8 +933,7 @@
 
 ;; Adds SPECIAL-Declarations at the beginning of a Body to *venv* .
 (defun %expand-special-declarations (body)
-  (multiple-value-bind (body-rest declarations)
-      (sys::parse-body body nil (vector *venv* *fenv*))
+  (multiple-value-bind (body-rest declarations) (sys::parse-body body)
     (declare (ignore body-rest)) ; do not throw away declarations!
     (let ((specials nil))
       (mapc #'(lambda (declspec)
@@ -988,14 +974,10 @@
                                   (eq (car declspec) 'SOURCE))))))))
       (values lambdabody nil) ; already expanded -> leave untouched
       (let ((*venv* *venv*))
-        (if blockp
-          (setq lambdabody
-                (cons (first lambdabody)
-                      (add-implicit-block name (rest lambdabody)))))
-        (values (list*
-                 (%expand-lambdalist (first lambdabody))
-                 (list 'DECLARE (list 'SOURCE lambdabody))
-                 (%expand-list (rest lambdabody)))
+        (if blockp (setq lambdabody (add-implicit-block name lambdabody)))
+        (values (list* (%expand-lambdalist (first lambdabody))
+                       (list 'DECLARE (list 'SOURCE lambdabody))
+                       (%expand-list (rest lambdabody)))
                 t)))))
 
 ;; expands a Lambda-list. returns 2 values.
@@ -1323,7 +1305,7 @@
                   ((:print *load-print*) *load-print*)
                   (if-does-not-exist t) (external-format ':default)
                   ((:echo *load-echo*) *load-echo*)
-                  ((:compiling *load-compiling*) *load-compiling*)
+                  ((:compiling *load-compiling*) *load-compiling* c-top)
                   ((:obsolete-action *load-obsolete-action*)
                    *load-obsolete-action*)
                   (extra-file-types '())
@@ -1363,12 +1345,19 @@
       (write-string " ..."))
     (when *load-compiling* (compiler::c-reset-globals))
     (sys::allow-read-eval input-stream t)
-    (block nil
+    ;; see `with-compilation-unit' -- `:compiling' sets a compilation unit
+    ;; the user might set `*load-compiling*' to T either directly
+    ;; or using the -C option, so we have to check that
+    ;; `*error-output*' et al are already bound
+    (progv (when (and *load-compiling*
+                      (or c-top (not (boundp '*error-count*))))
+             '(*error-count* *warning-count* *style-warning-count*))
+        '(0 0 0)
       (unwind-protect
            (tagbody weiter
              (when *load-echo* (fresh-line))
              (let ((obj (read input-stream nil end-of-file)))
-               (when (eql obj end-of-file) (return-from nil))
+               (when (eql obj end-of-file) (go done))
                (setq obj (multiple-value-list
                           (cond ((compiled-function-p obj) (funcall obj))
                                 (*load-compiling*
@@ -1376,12 +1365,12 @@
                                            obj)))
                                 (t (eval obj)))))
                (when *load-print* (when obj (print (first obj)))))
-             (go weiter))
+             (go weiter) done)
         (or (eq input-stream stream)
             (sys::built-in-stream-close input-stream))
         (or (eq stream filename)
-            (sys::built-in-stream-close stream))))
-    (when *load-compiling* (compiler::c-report-problems))
+            (sys::built-in-stream-close stream))
+        (when *load-compiling* (compiler::c-report-problems))))
     (when *load-verbose*
       (fresh-line)
       (write-string ";;")
@@ -1400,6 +1389,7 @@
 (sys::%putd 'defun              ; preliminary:
   (sys::make-macro
     (function defun (lambda (form env)
+      (declare (ignore env))
       (unless (and (consp (cdr form)) (consp (cddr form)))
         (error-of-type 'source-program-error
           (TEXT "~S: missing function name and/or parameter list")
@@ -1412,9 +1402,7 @@
           (error-of-type 'source-program-error
             (TEXT "~S: special operator ~S cannot be redefined.")
             'defun name))
-        (multiple-value-bind (body-rest declarations docstring)
-            (sys::parse-body body t env)
-          (declare (ignore docstring))
+        (multiple-value-bind (body-rest declarations) (sys::parse-body body t)
           #| `(PROGN
                (SYS::%PUT ',name 'SYS::DEFINITION
                  (CONS ',form (THE-ENVIRONMENT)))
@@ -1438,6 +1426,7 @@
 (sys::%putd 'do               ; preliminary definition of the macro DO
   (sys::make-macro
     (function do (lambda (form env)
+      (declare (ignore env))
       (let ((varclauselist (second form))
             (exitclause (third form))
             (body (cdddr form)))
@@ -1449,8 +1438,7 @@
               (reinitlist nil)
               (bodytag (gensym))
               (exittag (gensym)))
-          (multiple-value-bind (body-rest declarations)
-              (sys::parse-body body nil env)
+          (multiple-value-bind (body-rest declarations) (sys::parse-body body)
             (block do
               (tagbody 1
                 (when (atom varclauselist)
@@ -1504,12 +1492,12 @@
 (sys::%putd 'dotimes       ; preliminary Definition of the Macro DOTIMES
   (sys::make-macro
     (function dotimes (lambda (form env)
+      (declare (ignore env))
       (let ((var (first (second form)))
             (countform (second (second form)))
             (resultform (third (second form)))
             (body (cddr form)))
-        (multiple-value-bind (body-rest declarations)
-            (sys::parse-body body nil env)
+        (multiple-value-bind (body-rest declarations) (sys::parse-body body)
           (let ((g (gensym)))
             #| `(DO ((,var 0 (1+ ,var))
                      (,g ,countform))
@@ -1609,7 +1597,7 @@
             (lambdalist (caddr form))
             (body (cdddr form)))
         (multiple-value-bind (body-rest declarations docstring)
-            (sys::parse-body body t env)
+            (sys::parse-body body t)
           (let ((symbolform
                  (if (atom name)
                    `',name
